"""
The MIT License (MIT)
Copyright (c) Alwin Lohrie (Niwla23)
Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
Errors that can be thrown by the iservscrapping module.
"""
# pylint: disable=unnecessary-pass


class IservError(BaseException):
    """
    This exception is never actually thrown. All exceptions inherit from it so it can be used to catch all iserv errors.
    """
    pass


class LoginError(IservError):
    """
    This is raised when the login fails. Since iserv is to dumb to use http conventions, we need to check this by text.
    (It does not return anything but 200, even on not found or failed auth). Login therefore currently only works
     on german instances.
    """
    pass


class WrongContentError(IservError):
    """
    This is raised when the text the scrapper got is not the text it expected. Happens when wrong URLs are passed since
    Iserv does not use HTTP status codes (see LoginError)
    """
    pass



"""
The MIT License (MIT)
Copyright (c) Alwin Lohrie (Niwla23)
Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from typing import List
from dataclasses import dataclass
from bs4 import BeautifulSoup


from iservscrapping.base import BaseIserv
from iservscrapping.task import Task
from iservscrapping.untis_scrapper import UntisScrapper

numbers = ["1", "2", "3", "4", "5", "6", "7", "8", "9", "0"]


@dataclass
class Classtest:
    """Contains data about a classtest or event
    """
    date: str
    time: str
    name: str
    course: str
    kind: str


class Iserv(BaseIserv):
    """
    The main class you will want to use most of the time. Provides all the scrapping functions.

    This inherits from :exc:`base.BaseIserv`.
    """

    def get_home(self):
        """
        Returns:
            requests.Response: The response for the main page
        Gets the main page of iserv
        """
        return self.session.get(f"{self.url}/iserv")

    def get_untis_substitution_plan(self, path):
        """
        Args:
            path str: The path where the plan is located. Defaults to /iserv
        Returns:
            tuple: (plan dict, date str, week str) Returns the parsed plan, date and a or b week.
        Note: Make sure you have the raw plan, not a page with the plan in an iframe
        """
        res = self.session.get(self.url + path)
        res.raise_for_status()
        scrapper = UntisScrapper(res.text)
        return scrapper.scrape()

    def get_next_tests_formatted(self, path="/iserv"):
        """
        Args:
            path (:obj:`str`, optional): The path where the tests box is located. Defaults to /iserv
        Returns:
            list: Next tests from the box on mainpage.

        This does NOT parse the tasks into fields, it keeps the formatting and just splits into tests
        """

        test_list = []

        res = self.session.get(self.url + path)
        soup = BeautifulSoup(res.text, 'lxml')

        tests = soup.find("ul", "pl mb0")

        for i in tests.find_all("li"):
            test_list.append(i.text)
        return test_list

    def get_next_tests(self, path="/iserv") -> List[Classtest]:
        """
        Args:
            path (:obj:`str`, optional): The path where the tests box is located. Defaults to /iserv
        Returns:
            list: Next tests from the box on mainpage.
        """
        test_list = []

        res = self.session.get(self.url + path)
        soup = BeautifulSoup(res.text, 'lxml')

        tests = soup.find("ul", "pl mb0")

        for i in tests.find_all("li"):
            parsed = i.text.split(" - ")

            title_and_kind: List[str] = parsed[3].strip().split("(")

            entry = Classtest(
                date=parsed[0].strip(),
                time=parsed[1].strip(),
                course=parsed[2].strip(),
                kind=title_and_kind.pop()[:-1],
                name="(".join(title_and_kind)
            )
            test_list.append(entry)
        return test_list

    def get_tasks(self, path="/iserv/exercise?filter[status]=current"):
        """
        Args:
            path (:obj:`str`, optional): The path where the tasks are located.
                Defaults to /iserv/exercise?filter[status]=current
        Returns:
            list: List of Task objects for current user
        """

        tasks = []

        res = self.session.get(self.url + path)
        soup = BeautifulSoup(res.text, 'lxml')
        task_table = soup.find("table", {"id": "crud-table"})
        rows = task_table.find_all("tr")
        for row in rows:
            columns = row.find_all("td")
            if len(columns) == 7:
                current_task = Task(self.url, self.username, self.password)
                current_task.title = columns[0].find("a").text
                current_task.id = int(columns[0].find("a")[
                                      "href"].split("/")[-1])
                current_task.start_date = columns[1].text
                current_task.end_date = columns[2].text
                if columns[4].find("span", {"class": "glyphicon-ok"}):
                    current_task.done = True
                else:
                    current_task.done = False
                current_task.corrections = columns[5].text
                tasks.append(current_task)
        return tasks



"""
The MIT License (MIT)
Copyright (c) Alwin Lohrie (Niwla23)
Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from typing import List, Tuple
from dataclasses import dataclass
import re
from bs4 import BeautifulSoup, element
from iservscrapping.errors import WrongContentError


@dataclass
class SubstitutionEntry:
    """Represents one row in the substitution plan"""
    time: str
    subject: str
    teacher: str
    room: str
    courses: List[str]
    text: str | None


@dataclass
class CourseHeading:
    """Represents one Course in the substitution plan and contains the rows."""
    name: str
    entrys: List[SubstitutionEntry]

@dataclass
class SubstitutionPlanMetaTable:
    """Represents information from the table at the top of the plan"""
    teachers_absent: List[str]
    courses_absent: List[str]
    blocked_rooms: List[Tuple[str, str]]
    affected_teachers: List[str]
    affected_courses: List[str]
    affected_rooms: List[str]


@dataclass
class SubstitutionPlan:
    """Represents a parsed substitution plan document"""
    # pylint: disable=too-many-instance-attributes
    courses: List[CourseHeading]
    meta_table: SubstitutionPlanMetaTable
    date: str
    week: str


class UntisScrapper:
    """
    Args:
        html (str): The HTML to parse
    Attributes:
        html (str): The HTML to parse
        soup (str): The Soup of the HTML
        content_header_list (str): Ordered list of headers to match the fields from the table
    A scrapper to parse Untis substitution plans. you will most of the time want to use
    :exc:`Iserv.get_untis_substitution_plan`
    """

    # pylint: disable=too-few-public-methods

    def __init__(self, html: str):
        """Initializes the scraper

        Args:
            html (str): the html to parse
        """
        self.html = html
        self.soup = BeautifulSoup(html, 'lxml')
        self.content_header_list = [
            "time",
            "subject",
            "teacher",
            "text",
            "course",
            "room",
        ]

    def parse_content(self) -> List[CourseHeading]:
        """Parses the main content of the plan

        Returns:
            List[CourseHeading]: A list of CourseHeadings containing the actual data per course
        """
        current_class = "0b118b33-d1e4-4579-8055-fa5230d0c34b"
        parsed_plan = {}
        content_table = list(self.soup.select_one(".mon_list"))
        for row in content_table:
            if len(row) == 6:
                parsed_row = SubstitutionEntry(
                    time="", subject="", teacher="", courses=[], room="", text=None)
                if current_class not in parsed_plan.keys():
                    parsed_plan[current_class] = CourseHeading(current_class, [])
                for i, column in enumerate(row):
                    if i == 0:
                        parsed_row.time = column.text.strip()
                    elif i == 1:
                        parsed_row.subject = column.text.strip()
                    elif i == 2:
                        parsed_row.teacher = column.text.strip()
                    elif i == 3:
                        parsed_row.text = column.text.strip()
                    elif i == 4:
                        parsed_row.courses = column.text.strip()
                    elif i == 5:
                        parsed_row.room = column.text.strip()
                parsed_plan[current_class].entrys.append(parsed_row)
            elif len(row) == 1:
                first_element = list(row)[0]
                if isinstance(first_element, element.Tag):
                    current_class = list(row)[0].text.split(" ")[0]
        try:
            del parsed_plan['0b118b33-d1e4-4579-8055-fa5230d0c34b']
        except KeyError:
            pass

        return parsed_plan

    def parse_meta_table(self) -> SubstitutionPlanMetaTable:
        """parses the meta table at the top of the plan

        Returns:
            SubstitutionPlanMetaTable: The metadata
        """
        meta_table = list(self.soup.select_one("table.info"))
        meta_table_parsed = SubstitutionPlanMetaTable(
            teachers_absent=[],
            courses_absent=[],
            blocked_rooms=[],
            affected_teachers=[],
            affected_courses=[],
            affected_rooms=[]
        )

        for row in meta_table:
            try:
                key_name = list(row)[0].text.strip()
                value_raw = list(row)[1].text.strip()
            except IndexError:
                continue
            except AttributeError:
                continue
            if key_name == "Abwesende Lehrer":
                meta_table_parsed.teachers_absent = re.findall(
                    r"(..\. \S*)[,( \()]", value_raw)
            elif key_name == "Abwesende Klassen":
                meta_table_parsed.courses_absent = re.findall(
                    r"(\S*)\s\(", value_raw)
            elif key_name == "Blockierte Räume":
                meta_table_parsed.blocked_rooms = re.findall(
                    r"(\S*)\s\((\S*)\)", value_raw)
            elif key_name == "Betroffene Lehrer":
                meta_table_parsed.affected_teachers = [
                    i.strip() for i in value_raw.split(",")]
            elif key_name == "Betroffene Klassen":
                meta_table_parsed.affected_courses = [
                    i.strip() for i in value_raw.split(",")]
            elif key_name == "Betroffene Räume":
                meta_table_parsed.affected_rooms = [
                    i.strip() for i in value_raw.split(",")]

        return meta_table_parsed

    def scrape(self) -> SubstitutionPlan:
        """Parses the whole plan with all sections.

        Raises:
            WrongContentError: The HTML passed was not in the form expected.

        Returns:
            SubstitutionPlan: The whole substitution plan
        """
        try:
            header = self.soup.select_one(".mon_title").get_text().split(" ")
            date = header[0]
            week = header[-1]

        except AttributeError as error:
            raise WrongContentError(
                "Content was not parsable."
                "Since Iserv is to dumb to throw a 404, we can only guess about this.") from error

        parsed_content = self.parse_content()
        parsed_meta_table = self.parse_meta_table()

        result = SubstitutionPlan(
            courses=parsed_content, date=date, week=week, meta_table=parsed_meta_table)

        return result



"""
The MIT License (MIT)
Copyright (c) Alwin Lohrie (Niwla23)
Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
import logging

import requests
import requests_cache

from iservscrapping.errors import LoginError


class BaseIserv:
    """
    Args:
        url (str): The URL to connect to without trailing slash
        username (str): The Username to login with
        password (str): The Password to login with
        cache (bool): Whether to use cache or not. Defaults to False
    """

    # pylint: disable=too-few-public-methods

    def __init__(self, url, username, password, cache=False):
        self.username = username
        self.password = password
        self.time_to_next_substitution_request = 0
        self.url = url
        self.session = requests.Session()

        if cache:
            requests_cache.install_cache('cache', expire_after=3600)

    def login(self):
        """
        Raises:
            LoginError: The login failed.
        Sends a login request and stores the tokens in the session
        """
        logging.debug("Starting Login")
        headers = {'User-Agent': 'Mozilla/5.0'}
        payload = {'_username': self.username, '_password': self.password}
        response = self.session.post(self.url + '/iserv/app/login', headers=headers, data=payload)
        if "Anmeldung fehlgeschlagen!" in response.text:
            raise LoginError



"""
The MIT License (MIT)
Copyright (c) Alwin Lohrie (Niwla23)
Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from bs4 import BeautifulSoup
from iservscrapping.base import BaseIserv


class Task(BaseIserv):
    """
    Args:
        url (str): The URL to connect to without trailing slash
        username (str): The Username to login with
        password (str): The Password to login with
    Attributes:
        title (str): The title of the task
        start_date (str): The date and maybe time the task starts
        end_date (str): The date and maybe time the task ends
        done (str): Whether the task is done or not
        corrections (bool): If the task has been corrected by a teacher (broken)
        id (int): The internal ID of the task
        author (str): The user who created this task
        description (str): The description of the task
        provided_files (list): A list of files provided to solve the task

    Represents a task from the iserv task module.
    """

    # pylint: disable=too-many-instance-attributes
    # pylint: disable=invalid-name

    def __init__(self, url, username, password):
        super().__init__(url, username, password)
        self.title = None
        self.start_date = None
        self.end_date = None
        self.done = None
        self.corrections = None
        self.id = None

        self.author = None
        self.description = None
        self.provided_files = []

    def get_details(self, path="/iserv/exercise/show/"):
        """
        Args:
            path (:obj:`str`, optional): The path where the tests box is located. Defaults to /iserv/exercise/show/
        Returns:
            Task: The task filled with details
        Loads details for the task. For this, the page for that task needs to be loaded which is the reason that there
        is an extra function for it.
        """
        res = self.session.get(self.url + path + str(self.id))
        soup = BeautifulSoup(res.text, 'lxml')
        task_rows = soup.find("table", {"class": "table"}).find_all("tr")
        task_details = []
        for row in task_rows:
            task_details.append(row.find_all("td"))
        self.author = task_details[0][0].find("a").text
        self.description = task_details[3][0].text
        try:
            provided_files = task_details[5][0].find("ul").find_all("li")
            provided_files_list = []
            for file in provided_files:
                provided_files_dict = {"filename": file.find("a").text.split("\n")[0].strip(),
                                       "size": file.find("a").text.split("\n")[1].strip(),
                                       "link": file.find("a")["href"],
                                       }
                try:
                    provided_files_dict["description"] = file.text.split("\n")[1].strip().split("-")[1].strip()
                except IndexError:
                    provided_files_dict["description"] = ""
                provided_files_list.append(provided_files_dict)
            self.provided_files = provided_files_list
        except IndexError:
            pass
        return self



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['iservscrapping']

package_data = \
{'': ['*']}

install_requires = \
['Markdown>=3.3.6,<4.0.0',
 'beautifulsoup4>=4.10.0,<5.0.0',
 'lxml>=4.7.1,<5.0.0',
 'requests-cache>=0.9.1,<0.10.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'iservscrapping',
    'version': '6.0.0',
    'description': 'This Scrapper can get data from iserv servers hosting Untis plans. (Common in germany)',
    'long_description': None,
    'author': 'niwla23',
    'author_email': 'alwin@cloudserver.click',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""
The MIT License (MIT)
Copyright (c) Alwin Lohrie (Niwla23)
Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

import random
import time

import requests


# pylint: disable=no-self-use
# pylint: disable=duplicate-code
# pylint: disable=too-few-public-methods


class FakeBaseIserv:
    """
    Exactly the same members as BaseIserv, but no queries are made, everything is static.
    useful for development
    when you don't want to spam the server.
    """

    # pylint: disable=too-few-public-methods

    def __init__(self, url, username, password, cache=False):
        self.username = username
        self.password = password
        self.time_to_next_substitution_request = 0
        self.url = url
        self.session = requests.Session()
        if cache:
            pass

    def login(self):
        """
        Raises:
            LoginError: The login failed.
        Sends a login request and stores the tokens in the session
        """
        time.sleep(1)


class FakeTask(FakeBaseIserv):
    """
       Args:
           url (str): The URL to connect to without trailing slash
           username (str): The Username to login with
           password (str): The Password to login with
       Attributes:
           title (str): The title of the task
           start_date (str): The date and maybe time the task starts
           end_date (str): The date and maybe time the task ends
           done (str): Whether the task is done or not
           corrections (bool): If the task has been corrected by a teacher (broken)
           id (int): The internal ID of the task
           author (str): The user who created this task
           description (str): The description of the task
           provided_files (list): A list of files provided to solve the task

       Same as Task, but loads only fake data, no requests made. Useful for development
       """

    # pylint: disable=too-many-instance-attributes
    # pylint: disable=invalid-name

    def __init__(self, url, username, password):
        super().__init__(url, username, password)
        self.title = None
        self.start_date = None
        self.end_date = None
        self.done = None
        self.corrections = None
        self.id = None

        self.author = None
        self.description = None
        self.provided_files = []

    def get_details(self, path="/iserv/exercise/show/"):
        """
        Args:
            path (:obj:`str`, optional): The path where the tests box is located.
                Defaults to /iserv/exercise/show/
        Returns:
            Task: The task filled with details
        Loads details for the task. For this, the page for that task needs to be loaded which
        is the reason that there
        is an extra function for it.
        """
        # Do something with it
        path.split("x")
        self.author = random.choice(["Mr X.", "Mrs. Y"])
        self.description = random.choice(
            ["Do this task.", "This is a fake task!", "hi pls do your task"])

        provided_files_list = []
        for _ in range(1, random.randint(2, 8)):
            filename = random.choice(
                ["coolfile.pdf", "filename with spaces", "I am a file lolä.png"])
            provided_files_dict = \
                {
                    "filename": filename,
                    "size": f"{random.randint(1, 50)}kb",
                    "link": "/path/to/the/file",
                    "description": "file description"
                }

            provided_files_list.append(provided_files_dict)
        self.provided_files = provided_files_list
        return self


class FakeIserv(FakeBaseIserv):
    """
    Exactly the same members as Iserv, but no queries are made, everything is static.
    useful for development
    when you don't want to spam the server.
    """

    def get_home(self):
        """
        Returns:
            requests.Response: The response for the main page
        Gets the main page of iserv
        """
        return "<html>This is not implemented correctly due to copyright.</html>"

    def get_untis_substitution_plan(self, path):
        """
        Args:
            path str: The path where the plan is located. Defaults to /iserv
        Returns:
            tuple: (plan dict, date str, week str) Returns the parsed plan, date and a or b week.
        Note: Make sure you have the raw plan, not a page with the plan in an iframe
        """
        path.split("x")
        return {
            "-----": [
                {
                    "course": "",
                    "room": "",
                    "subject": "BER",
                    "teacher": "XXX→XXX",
                    "text": "",
                    "time": "1",
                },
                {
                    "course": "",
                    "room": "H-1",
                    "subject": "",
                    "teacher": "XXX→XXX",
                    "text": "",
                    "time": "2/3",
                },
                {
                    "course": "",
                    "room": "A2",
                    "subject": "",
                    "teacher": "XXX→XXX",
                    "text": "",
                    "time": "4/5",
                },
            ],
            "10a": [
                {
                    "course": "10af",
                    "room": "---",
                    "subject": "RE",
                    "teacher": "XXX",
                    "text": "",
                    "time": "1 - 2",
                }
            ],
            "10b": [
                {
                    "course": "10b",
                    "room": "---",
                    "subject": "DE",
                    "teacher": "XXX",
                    "text": "",
                    "time": "5",
                },
                {
                    "course": "10b",
                    "room": "---",
                    "subject": "DE",
                    "teacher": "XXX",
                    "text": "",
                    "time": "6",
                },
            ],
            "10f": [
                {
                    "course": "10af",
                    "room": "---",
                    "subject": "RE",
                    "teacher": "XXX",
                    "text": "",
                    "time": "1 - 2",
                }
            ],
            "5c": [
                {
                    "course": "5c",
                    "room": "B1.04",
                    "subject": "PH",
                    "teacher": "XXX",
                    "text": "KA",
                    "time": "3 - 4",
                }
            ],
            "6d": [
                {
                    "course": "6d",
                    "room": "B2.04",
                    "subject": "VFG→EK",
                    "teacher": "HST→XXX",
                    "text": "",
                    "time": "2",
                }
            ],
            "8f": [
                {
                    "course": "8f",
                    "room": "PO2",
                    "subject": "CH→SN",
                    "teacher": "XXX",
                    "text": "",
                    "time": "3 - 4",
                }
            ],
            "9c": [
                {
                    "course": "9c",
                    "room": "DE2",
                    "subject": "CH→BI",
                    "teacher": "XXX→XXX",
                    "text": "",
                    "time": "1",
                },
                {
                    "course": "9c",
                    "room": "DE2",
                    "subject": "CH→BI",
                    "teacher": "XXX→XXX",
                    "text": "",
                    "time": "2",
                },
            ],
            "Q1-L2-MA-XXX": [
                {
                    "course": "Q1-L2-MA-XXX",
                    "room": "---",
                    "subject": "MA",
                    "teacher": "XXX",
                    "text": "",
                    "time": "3 - 4",
                }
            ],
            "Q1-L4-de-XXX": [
                {
                    "course": "Q1-L4-de-XXX",
                    "room": "---",
                    "subject": "DE",
                    "teacher": "XXX",
                    "text": "",
                    "time": "5 - 6",
                }
            ],
            "Q2-L2-BI-XXX": [
                {
                    "course": "Q2-L2-BI-XXX",
                    "room": "NW1→HÖ",
                    "subject": "BI",
                    "teacher": "XXX",
                    "text": "",
                    "time": "3 - 4",
                }
            ],
            "Q2-L2-MU-XXX": [
                {
                    "course": "Q2-L2-MU-XXX",
                    "room": "---",
                    "subject": "MU",
                    "teacher": "XXX",
                    "text": "",
                    "time": "3 - 4",
                }
            ],
            "Q2-L4-bi-XXX": [
                {
                    "course": "Q2-L4-bi-XXX",
                    "room": "---",
                    "subject": "BI",
                    "teacher": "XXX",
                    "text": "",
                    "time": "5 - 6",
                }
            ],
            "Q2-L4-ma-XXX": [
                {
                    "course": "Q2-L4-ma-XXX",
                    "room": "---",
                    "subject": "MA",
                    "teacher": "XXX",
                    "text": "",
                    "time": "5 - 6",
                }
            ],
        }, "3.11.2020", "A"

    def get_next_tests_formatted(self, path="/iserv"):
        """
        Args:
            path (:obj:`str`, optional): The path where the tests box is located. Defaults to /iserv
        Returns:
            list: Next tests from the box on mainpage.

        This does NOT parse the tasks into fields, it keeps the formatting and just splits
            into tests
        """
        path.split("x")
        return ['Mi, 4.11. - 11:30 - Klasse 9F\n                        - 9f Erdkunde XXX',
                'Mi, 11.11. - 09:40 - Klasse 9F\n                        - 9f Sn XXX',
                'Fr, 13.11. - 09:40 - Klasse 9F\n                        - Biologie XXX',
                'Mo, 16.11. - 09:40 - Klasse 9F\n                        - 9F GE XXX',
                'Do, 26.11. - 09:40 - Klasse 9F\n                        - 9f Ku XXX',
                'Fr, 4.12. - 11:30 - Klasse 9F\n                        - 9f PW XXX',
                'Do, 10.12. - 11:30 - Klasse 9F\n                        - 9F Mathematik 2 XXX',
                'Di, 15.12. - 11:30 - Klasse 9F\n                        - 9F Chemie XXX',
                'Fr, 18.12. - 07:50 - Klasse 9F\n                        - 9f Sn XXX']

    def get_next_tests(self, path="/iserv"):
        """
        Args:
            path (:obj:`str`, optional): The path where the tests box is located. Defaults to /iserv
        Returns:
            list: Next tests from the box on mainpage.
        """
        path.split("x")
        tests = []
        for _ in range(1, random.randint(1, 10)):
            tests.append(
                {
                    'date': random.choice(['Mi, 4.11.', 'Do, 5.11.', 'Fr, 6.11.']),
                    'time': random.choice(['11:30', '9:20', '7:50']),
                    'class': random.choice(['Klasse 9F', 'Klasse 10A', 'Klasse 5D']),
                    'subject': random.choice(['9f Erdkunde XXX', '10a Mathe XXX']) + str(
                        random.randint(1, 100))
                }
            )
        return tests

    def get_tasks(self, path="/iserv/exercise?filter[status]=current"):
        """
        Args:
            path (:obj:`str`, optional): The path where the tasks are located.
                Defaults to /iserv/exercise?filter[status]=current
        Returns:
            list: List of Task objects for current user
        """

        path.split("x")
        tasks = []

        for _ in range(0, random.randint(0, 15)):
            task = FakeTask(self.url, self.username, self.password)
            task.title = random.choice(
                ["Tasks for next week", "Hey i made a task", "Home-work tasks"])
            task.id = random.randint(1, 5000)
            task.start_date = "22.03.2020"
            task.end_date = "26.03.2020 18:00"
            task.done = random.choice([True, False])
            task.corrections = "not implemented"
            tasks.append(task)
        return tasks
